<?php

namespace Buyesrfolio\Claim\Http\Controllers;

use App\Http\Controllers\APIController as BaseController;
use Buyesrfolio\Claim\Http\Requests\ClaimRequest;
use Buyesrfolio\Claim\Interfaces\ClaimRepositoryInterface;
use Buyesrfolio\Claim\Models\Claim;
use Buyesrfolio\Claim\Forms\Claim as Form;

/**
 * APIController  class for claim.
 */
class ClaimAPIController extends BaseController
{

    /**
     * Initialize claim resource controller.
     *
     * @param type ClaimRepositoryInterface $claim
     *
     * @return null
     */
    public function __construct(ClaimRepositoryInterface $claim)
    {
        parent::__construct();
        $this->repository = $claim;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Buyesrfolio\Claim\Repositories\Criteria\ClaimResourceCriteria::class);
    }

    /**
     * Display a list of claim.
     *
     * @return Response
     */
    public function index(ClaimRequest $request)
    {
        return $this->repository
            ->setPresenter(\Buyesrfolio\Claim\Repositories\Presenter\ClaimPresenter::class)
            ->paginate();
    }

    /**
     * Display claim.
     *
     * @param Request $request
     * @param Model   $claim
     *
     * @return Response
     */
    public function show(ClaimRequest $request, Claim $claim)
    {
        return $claim->setPresenter(\Buyesrfolio\Claim\Repositories\Presenter\ClaimListPresenter::class);
        ;
    }

    /**
     * Create new claim.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(ClaimRequest $request)
    {
        try {
            $data              = $request->all();
            $data['user_id']   = user_id();
            $data['user_type'] = user_type();
            $data              = $this->repository->create($data);
            $message           = trans('messages.success.created', ['Module' => trans('claim::claim.name')]);
            $code              = 204;
            $status            = 'success';
            $url               = guard_url('claim/claim/' . $claim->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('claim/claim');
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Update the claim.
     *
     * @param Request $request
     * @param Model   $claim
     *
     * @return Response
     */
    public function update(ClaimRequest $request, Claim $claim)
    {
        try {
            $data = $request->all();

            $claim->update($data);
            $message = trans('messages.success.updated', ['Module' => trans('claim::claim.name')]);
            $code    = 204;
            $status  = 'success';
            $url     = guard_url('claim/claim/' . $claim->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('claim/claim/' . $claim->getRouteKey());
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Remove the claim.
     *
     * @param Model   $claim
     *
     * @return Response
     */
    public function destroy(ClaimRequest $request, Claim $claim)
    {
        try {
            $claim->delete();
            $message = trans('messages.success.deleted', ['Module' => trans('claim::claim.name')]);
            $code    = 202;
            $status  = 'success';
            $url     = guard_url('claim/claim/0');
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('claim/claim/' . $claim->getRouteKey());
        }
        return compact('message', 'code', 'status', 'url');
    }

    /**
     * Return the form elements as json.
     *
     * @param String   $element
     *
     * @return json
     */
    public function form($element = 'fields')
    {
        $form = new Form();
        return $form->form($element, true);
    }

}
