<?php

namespace Buyesrfolio\Claim\Policies;

use Litepie\User\Contracts\UserPolicy;
use Buyesrfolio\Claim\Models\Claim;

class ClaimPolicy
{

    /**
     * Determine if the given user can view the claim.
     *
     * @param UserPolicy $user
     * @param Claim $claim
     *
     * @return bool
     */
    public function view(UserPolicy $user, Claim $claim)
    {
        if ($user->canDo('claim.claim.view') && $user->isAdmin()) {
            return true;
        }

        return $claim->user_id == user_id() && $claim->user_type == user_type();
    }

    /**
     * Determine if the given user can create a claim.
     *
     * @param UserPolicy $user
     * @param Claim $claim
     *
     * @return bool
     */
    public function create(UserPolicy $user)
    {
        return  $user->canDo('claim.claim.create');
    }

    /**
     * Determine if the given user can update the given claim.
     *
     * @param UserPolicy $user
     * @param Claim $claim
     *
     * @return bool
     */
    public function update(UserPolicy $user, Claim $claim)
    {
        if ($user->canDo('claim.claim.edit') && $user->isAdmin()) {
            return true;
        }

        return $claim->user_id == user_id() && $claim->user_type == user_type();
    }

    /**
     * Determine if the given user can delete the given claim.
     *
     * @param UserPolicy $user
     * @param Claim $claim
     *
     * @return bool
     */
    public function destroy(UserPolicy $user, Claim $claim)
    {
        return $claim->user_id == user_id() && $claim->user_type == user_type();
    }

    /**
     * Determine if the given user can verify the given claim.
     *
     * @param UserPolicy $user
     * @param Claim $claim
     *
     * @return bool
     */
    public function verify(UserPolicy $user, Claim $claim)
    {
        if ($user->canDo('claim.claim.verify')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the given user can approve the given claim.
     *
     * @param UserPolicy $user
     * @param Claim $claim
     *
     * @return bool
     */
    public function approve(UserPolicy $user, Claim $claim)
    {
        if ($user->canDo('claim.claim.approve')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the user can perform a given action ve.
     *
     * @param [type] $user    [description]
     * @param [type] $ability [description]
     *
     * @return [type] [description]
     */
    public function before($user, $ability)
    {
        if ($user->isSuperuser()) {
            return true;
        }
    }
}
