<?php

namespace Buyesrfolio\Claim\Providers;

use Illuminate\Support\ServiceProvider;

class ClaimServiceProvider extends ServiceProvider
{
    /**
     * Indicates if loading of the provider is deferred.
     *
     * @var bool
     */
    protected $defer = false;

    /**
     * Bootstrap the application events.
     *
     * @return void
     */
    public function boot()
    {
        // Load view
        $this->loadViewsFrom(__DIR__ . '/../../resources/views', 'claim');

        // Load translation
        $this->loadTranslationsFrom(__DIR__ . '/../../resources/lang', 'claim');

        // Load migrations
        $this->loadMigrationsFrom(__DIR__ . '/../../database/migrations');

        // Call pblish redources function
        $this->publishResources();

    }

    /**
     * Register the service provider.
     *
     * @return void
     */
    public function register()
    {
        $this->mergeConfig();
        $this->registerClaim();
        $this->registerFacade();
        $this->registerBindings();
        //$this->registerCommands();
    }


    /**
     * Register the application bindings.
     *
     * @return void
     */
    protected function registerClaim()
    {
        $this->app->bind('claim', function($app) {
            return new Claim($app);
        });
    }

    /**
     * Register the vault facade without the user having to add it to the app.php file.
     *
     * @return void
     */
    public function registerFacade() {
        $this->app->booting(function()
        {
            $loader = \Illuminate\Foundation\AliasLoader::getInstance();
            $loader->alias('Claim', 'Lavalite\Claim\Facades\Claim');
        });
    }

    /**
     * Register bindings for the provider.
     *
     * @return void
     */
    public function registerBindings() {
        // Bind facade
        $this->app->bind('buyesrfolio.claim', function ($app) {
            return $this->app->make('Buyesrfolio\Claim\Claim');
        });

                // Bind Claim to repository
        $this->app->bind(
            'Buyesrfolio\Claim\Interfaces\ClaimRepositoryInterface',
            \Buyesrfolio\Claim\Repositories\Eloquent\ClaimRepository::class
        );

        $this->app->register(\Buyesrfolio\Claim\Providers\AuthServiceProvider::class);
        
        $this->app->register(\Buyesrfolio\Claim\Providers\RouteServiceProvider::class);
            }

    /**
     * Merges user's and claim's configs.
     *
     * @return void
     */
    protected function mergeConfig()
    {
        $this->mergeConfigFrom(
            __DIR__ . '/../../config/config.php', 'buyesrfolio.claim'
        );
    }

    /**
     * Register scaffolding command
     */
    protected function registerCommands()
    {
        if ($this->app->runningInConsole()) {
            $this->commands([
                Commands\MakeClaim::class,
            ]);
        }
    }
    /**
     * Get the services provided by the provider.
     *
     * @return array
     */
    public function provides()
    {
        return ['buyesrfolio.claim'];
    }

    /**
     * Publish resources.
     *
     * @return void
     */
    private function publishResources()
    {
        // Publish configuration file
        $this->publishes([__DIR__ . '/../../config/config.php' => config_path('buyesrfolio/claim.php')], 'config');

        // Publish admin view
        $this->publishes([__DIR__ . '/../../resources/views' => base_path('resources/views/vendor/claim')], 'view');

        // Publish language files
        $this->publishes([__DIR__ . '/../../resources/lang' => base_path('resources/lang/vendor/claim')], 'lang');

        // Publish public files and assets.
        $this->publishes([__DIR__ . '/public/' => public_path('/')], 'public');
    }
}
